@@ -1,4 +1,4 @@
-import xml.etree.ElementTree as et
+from lxml import etree
 
 
 ###########################################################################
@@ -9,20 +9,21 @@ import xml.etree.ElementTree as et
 
 
 # will return a list of nodes specified by an attribute key and an attribute value from a parent node
-def get_child_nodes(node_parent, node_tag, node_att_name=None, node_att_val=None):
+def get_child_nodes(node_parent, node_tag, node_att_name=None, node_att_val=None, namespace_map=None):
     selector = node_tag
-    return find_all_nodes(node_parent, node_tag, node_att_name, node_att_val, recursive=False)
+    return find_all_nodes(node_parent, node_tag, node_att_name, node_att_val, recursive=False, namespace_map=namespace_map)
 
 
 # will search in all of the direct children of the root
-def get_root_direct_child_nodes(xml_file, node_tag, node_att_name=None, node_att_val=None):
-    return get_child_nodes(get_root_node(xml_file), node_tag, node_att_name, node_att_val)
+def get_root_direct_child_nodes(xml_file, node_tag, node_att_name=None, node_att_val=None, namespace_map=None):
+    return get_child_nodes(get_root_node(xml_file), node_tag, node_att_name, node_att_val, namespace_map)
 
 
 # will return a list of nodes which doesn't have a specific attribute
-def get_nodes_from_xml_without_att(xml_file, node_tag, node_att_name=None):
+def get_nodes_from_xml_without_att(xml_file, node_tag, node_att_name=None, namespace_map=None):
     root = xml_file.getroot()
     relevant_nodes = []
+    node_att_name = _sanitize_att_name(node_att_name, namespace_map)
     nodes = root.findall(node_tag)
     for node in nodes:
         if get_node_att(node, node_att_name) is None:
@@ -83,17 +84,21 @@ def set_node_text(node, text):
 
 
 # will return the value of a given att from a desired node
-def get_node_att(node, att_name):
+def get_node_att(node, att_name, namespace_map=None):
+    att_name = _sanitize_att_name(att_name, namespace_map)
     return node.get(att_name)
 
 
-# will return an xml file
-def read_xml_file(xml_path, namespace_dict=None):
-    xml = et.parse(xml_path)
+# will read and return an xml file.
+# added a custom parser to prevent the commends from being removed
+def read_xml_file(xml_path, namespace_dict=None, remove_comments=False):
+    parser = etree.XMLParser(remove_comments=remove_comments)
+    tree = etree.parse(xml_path, parser=parser)
+    etree.set_default_parser(parser)
     if namespace_dict:
         for prefix, uri in namespace_dict.items():
-            et.register_namespace(prefix, uri)
-    return xml
+            etree.register_namespace(prefix, uri)
+    return tree
 
 
 # will save the changes made in an xml file
@@ -109,21 +114,18 @@ def create_xml_file(root_node_tag, output_file):
     xml = et.Element(root_node_tag)
     tree = et.ElementTree(xml)
     save_xml_file(tree, output_file)
+    # tree = read_xml_file(output_file)   # maybe to read the xml again, to prevent the comments from being removed?
     return tree
 
 
-def create_and_add_new_node(parent_node, node_tag, att_dict=None, node_text=None):
-    """
-    Will add a node to a certain (relative) location
+# will add a node to a relative location
+def create_and_add_new_node(parent_node, node_tag, att_val_dict=None, node_text=None, namespace_map=None):
+    node_att_val_dict = {}
+    if att_val_dict is not None and namespace_map is not None:
+        for key, val in att_val_dict.items():
+            node_att_val_dict[_sanitize_att_name(key, namespace_map)] = val
 
-    Args:
-        param xml: the xml file
-        param parent_node: the parent of the node to add (for root, leave blank)
-        param node_tag: the tag of the node ('String', for example)
-    """
-    if att_dict is None:
-        att_dict = {}
-    node = et.SubElement(parent_node, node_tag, att_dict)
+    node = etree.SubElement(parent_node, node_tag, node_att_val_dict)
     if node_text is not None:
         set_node_text(node, node_text)
     return node
@@ -144,12 +146,6 @@ def add_nodes(parent_node, child_nodes):
         add_node(parent_node, child_node)
 
 
-# will add attribute to a given node
-def set_node_atts(node, atts_dict):
-    for key, val in atts_dict.items():
-        node.set(key, val)
-
-
 # will merge xml1 with xml2 and return a new xml comprising both of them.
 # NOTICE: this function will compare the direct root child nodes and merge/append them.
 def merge_xml1_with_xml2(xml1, xml2):
@@ -163,7 +159,7 @@ def merge_xml1_with_xml2(xml1, xml2):
                 parent_node = xml2_child
                 break
 
-        if parent_node:
+        if parent_node is not None:
             xml1_direct_children = get_all_direct_child_nodes(xml1_child)
             add_nodes(parent_node, xml1_direct_children)
 
@@ -174,11 +170,11 @@ def merge_xml1_with_xml2(xml1, xml2):
 
 
 # Will turn a simple dictionary to an xml file, by hierarchical order
-def simple_dict_to_xml(xml_dict, root_name, output_path):
+def simple_dict_to_xml(xml_dict, root_name, output_path, namespace_map=None):
     # will unpack the parent recursively
     def recursive_unpack_parent(parent_dict, parent=None):
         for key, val in parent_dict.items():
-            parent = create_and_add_new_node(parent, xml, key)
+            parent = create_and_add_new_node(parent, xml, key, namespace_map=namespace_map)
             if type(val) is dict:
                 recursive_unpack_parent(val, parent)
 
@@ -191,10 +187,20 @@ def get_root_node(xml_file):
     return xml_file.getroot()
 
 
+# will add attribute to a given node.
+# If adding a namespace map, add, for example {'android', 'http://schemas.android.com/apk/res/android'}. The keys should be normal. Like: {"android:value": xxxx}
+def set_node_atts(node, atts_dict, namespace_map=None):
+    for key, val in atts_dict.items():
+        key = _sanitize_att_name(key, namespace_map)
+        node.set(key, val)
+
+
 # will search for a node
-def find_all_nodes(parent_node, node_tag, node_att_name=None, node_att_val=None, recursive=True):
+def find_all_nodes(parent_node, node_tag, node_att_name=None, node_att_val=None, recursive=True, namespace_map=None):
+
     selector = node_tag
     if node_att_name is not None:
+        node_att_name = _sanitize_att_name(node_att_name, namespace_map)
         if node_att_val is not None:
             selector = node_tag + "/[@" + node_att_name + "='" + node_att_val + "']"
         else:
@@ -202,4 +208,14 @@ def find_all_nodes(parent_node, node_tag, node_att_name=None, node_att_val=None,
     if recursive:
         return parent_node.findall(f'.//{selector}')
     else:
-        return parent_node.findall(selector)
+        return parent_node.findall(selector, namespace_map)
+
+
+# will parse the namespace, as required, in the att_name
+def _sanitize_att_name(att_name, namespace_map):
+    if namespace_map is not None:
+        for prefix, uri in namespace_map.items():
+            if prefix in att_name:
+                att_name =  '{' + str(att_name).replace(':','}').replace(prefix, uri)
+                break
+    return att_name
\ No newline at end of file
